/*
 * $Id: background.h 482 2006-04-12 09:00:23Z satoken $
 *
 * PHMMTS -- an implementation of Pair Hidden Markov Models on Tree Structures,
 *           which is based on "Pair Hidden Markov Models on Tree Structures",
 *           Yasubumi Sakakibara, ISMB 2003.
 *
 * Copyright (C) 2003-2006 Kengo Sato
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef __INC_BACKGROUND_H__
#define __INC_BACKGROUND_H__

#include <vector>
#include "defines.h"

namespace PHMMTS {

  template < class cost_type >
  class Background
  {
  private:
    int from_;
    std::vector<cost_type> ins_table_;
    std::vector<cost_type> del_table_;

  public:
    Background() : from_(-1), ins_table_(), del_table_() { }

    template < class CT, class STree, class Sequence >
    void initialize(const CT& cost_table,
		    const STree& tree, const Sequence& rna,
		    int from, int to);

    template < class CT, class Sequence >
    void update_position(const CT& cost_table,
			 const Sequence& rna, int from, int to);

    cost_type inside(const Position& pos) const;
    cost_type outside(const Position& pos) const;
  };
};

#endif // __INC_BACKGROUND_H__

// Local Variables:
// mode: C++
// End:
