/*
 * $Id: cyk_no_trace.h 482 2006-04-12 09:00:23Z satoken $
 *
 * PHMMTS -- an implementation of Pair Hidden Markov Models on Tree Structures,
 *           which is based on "Pair Hidden Markov Models on Tree Structures",
 *           Yasubumi Sakakibara, ISMB 2003.
 *
 * Copyright (C) 2003-2006 Kengo Sato
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef __INC_CYK_NO_TRACE_H__
#define __INC_CYK_NO_TRACE_H__

#include "defines.h"
#include "cell.h"
#include "background.h"

namespace PHMMTS {
  template <class> class DPTable;
  template <class, template <class> class > class AlgorithmImpl;
  class ProgressBar;
  
  template <class CostTable_, template <class> class DPTable_ = DPTable>
  class CYKnoTrace
  {
    friend class AlgorithmImpl<CostTable_,DPTable_>;
  public:
    typedef typename CostTable_::cost_type cost_type;

  private:
    typedef CYKnoTraceCell<cost_type> cell_type;
    typedef DPTable_< cell_type > pcyk_table;
    typedef Background< cost_type > bk_table;
    typedef AlgorithmImpl<CostTable_,DPTable_> Impl;

  private:
    pcyk_table pcyk_;
    bk_table bk_;
    Position max_pos_;
    cost_type max_;
    Impl* impl_;
    search_type_t search_type_;
    ProgressBar* progress_;

  public:
    CYKnoTrace(const CostTable_& cost_table, search_type_t search_type,
	       ProgressBar* progress = NULL, int max_length=0);

    ~CYKnoTrace();

    Position pos() const { return max_pos_; }
    cost_type cost() const { return max_; }

    template <class STree, class Sequence>
    cost_type run(const STree& tree, const Sequence& rna, int from, int to);

    template <class STree, class Sequence>
    cost_type run(const STree& tree, const Sequence& rna)
    {
      return run(tree, rna, 0, rna.size());
    }

    template <class STree, class Sequence>
    cost_type scan_first(const STree& tree, const Sequence& rna,
			 int from, int to);

    template <class STree, class Sequence>
    cost_type scan_next(const STree& tree, const Sequence& rna,
			int from, int to, int pos);

    template < class SkeletalTree_ >
    cost_type local_cost(const SkeletalTree_& skel) const
    {
      return 1.0;
    }

  private:
    template < class CostType, class Position, class STree, class Seq >
    void operator()(node_type_t type, const CostType& emit,
		    const Position& parent,
		    const STree& tree, const Seq& rna);

    template < class CostType, class Position, class STree, class Seq >
    void operator()(node_type_t type,
		    const CostType& emit, const CostType& state,
		    const Position& parent, const Position& child,
		    const STree& tree, const Seq& rna);

    template < class CostType, class Position, class STree, class Seq >
    void operator()(node_type_t type,
		    const CostType& emit, const CostType& state, 
		    const Position& parent,
		    const Position& lchild, const Position& rchild,
		    const STree& tree, const Seq& rna);
  };
};

#endif // __INC_CYK_NO_TRACE_H__

// Local Variables:
// mode: C++
// End:
