/*
 * $Id: cyktable.h 482 2006-04-12 09:00:23Z satoken $
 *
 * PHMMTS -- an implementation of Pair Hidden Markov Models on Tree Structures,
 *           which is based on "Pair Hidden Markov Models on Tree Structures",
 *           Yasubumi Sakakibara, ISMB 2003.
 *
 * Copyright (C) 2003-2006 Kengo Sato
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#ifndef __INC_CYKTABLE_H__
#define __INC_CYKTABLE_H__

#include <cassert>

/**
 * @brief A templete class of CYK tables
 *        used for Cocke-Younger-Kasami (CYK) algorithm
 */
template <class T>
class CYKTable
{
public:
  typedef T value_type;

private:
  T* table_;
  T** ptr_;
  size_t size_;
  
public:
  CYKTable(size_t size) : table_(NULL), ptr_(NULL), size_(size)
  {
    table_ = new T[(size_+1)*size_/2];
    ptr_ = new T*[size_];
    for (size_t i=0; i<size_; ++i)
      ptr_[i]=&table_[i*size_-(i+1)*i/2];
  }

  ~CYKTable() { delete[] table_; delete[] ptr_; }

  size_t size() const { return size_; }

  void fill(const T& val)
  {
    for (size_t i=0; i<(size_+1)*size_/2; ++i)
      table_[i] = val;
  }

  void put(int i, int j, const T& val)
  {
    assert(i<=j);
    ptr_[i][j]=val;
  }

  const T& get(int i, int j) const
  {
    assert(i<=j);
    return ptr_[i][j];
  }

  T& get(int i, int j)
  {
    assert(i<=j);
    return ptr_[i][j];
  }
  const T& operator()(int i, int j) const
  {
    return get(i,j);
  }
  
  T& operator()(int i, int j)
  {
    return get(i,j);
  }
};

#endif // __INC_CYKTABLE_H__

// Local Variables:
// mode: C++
// End:
