/*
 * $Id: defaultcosttable.cpp 482 2006-04-12 09:00:23Z satoken $
 *
 * PHMMTS -- an implementation of Pair Hidden Markov Models on Tree Structures,
 *           which is based on "Pair Hidden Markov Models on Tree Structures",
 *           Yasubumi Sakakibara, ISMB 2003.
 *
 * Copyright (C) 2003-2006 Kengo Sato
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <climits>
#include "defaultcosttable.h"

using namespace PHMMTS;

/*
  this score table is based on "Finding the most significant common 
  sequence and structure motifs in a set of RNA sequences",
  Gorodkin, J. et al., Nucleic Acids Research, 25, 3724-32, 1997
*/
static
const DefaultCostTable::cost_type::value_type
DEFAULT_EMIT_COST[5][5][5][5] = {
  { // A  A  A  A  A    C  C  C  C  C    G  G  G  G  G    U  U  U  U  U    -  -  -  -  -    
    // A  C  G  U  -    A  C  G  U  -    A  C  G  U  -    A  C  G  U  -    A  C  G  U  -    
    {{ 2,-2,-2,-2,-3},{-2,-6,-6,-6,-7},{-2,-6,-6,-6,-7},{-2,-6,-6,-6,-7},{-3,-7,-7,-7,-8}}, // A A
    {{-2, 2,-2,-2,-3},{-6,-2,-6,-6,-7},{-6,-2,-6,-6,-7},{-6,-2,-6,-6,-7},{-7,-3,-7,-7,-8}}, // A C
    {{-2,-2, 2,-2,-3},{-6,-6,-2,-6,-7},{-6,-6,-2,-6,-7},{-6,-6,-2,-6,-7},{-7,-7,-3,-7,-8}}, // A G
    {{-2,-2,-2, 4,-3},{-6,-6, 3,-2,-7},{-6, 3,-6, 3,-7},{ 3,-6, 3,-2,-7},{-7,-7,-7,-3,-8}}, // A U
    {{-3,-3,-3,-3, 1},{-7,-7,-7,-7,-3},{-7,-7,-7,-7,-3},{-7,-7,-7,-7,-3},{-8,-8,-8,-8,-4}}  // A -
  },
  { // A  A  A  A  A    C  C  C  C  C    G  G  G  G  G    U  U  U  U  U    -  -  -  -  -    
    // A  C  G  U  -    A  C  G  U  -    A  C  G  U  -    A  C  G  U  -    A  C  G  U  -    
    {{-2,-6,-6,-6,-7},{ 2,-2,-2,-2,-3},{-2,-6,-6,-6,-7},{-2,-6,-6,-6,-7},{-3,-7,-7,-7,-8}}, // C A
    {{-6,-2,-6,-6,-7},{-2, 2,-2,-2,-3},{-6,-2,-6,-6,-7},{-6,-2,-6,-6,-7},{-3,-7,-7,-7,-8}}, // C C
    {{-6,-6,-2, 3,-7},{-2,-2, 4,-2,-3},{-6, 3,-2, 3,-7},{ 3,-6, 3,-6,-7},{-7,-7,-3,-7,-8}}, // C G
    {{-6,-6,-6,-2,-7},{-2,-2,-2, 2,-3},{-6,-6,-6,-2,-7},{-6,-6,-6,-2,-7},{-7,-7,-7,-3,-8}}, // C U
    {{-7,-7,-7,-7,-3},{-3,-3,-3,-3, 1},{-7,-7,-7,-7,-3},{-7,-7,-7,-7,-3},{-8,-8,-8,-8,-4}}  // C -
  },
  { // A  A  A  A  A    C  C  C  C  C    G  G  G  G  G    U  U  U  U  U    -  -  -  -  -    
    // A  C  G  U  -    A  C  G  U  -    A  C  G  U  -    A  C  G  U  -    A  C  G  U  -    
    {{-2,-6,-6,-6,-7},{-2,-6,-6,-6,-7},{ 2,-2,-2,-2,-3},{-2,-6,-6,-6,-7},{-3,-7,-7,-7,-8}}, // G A
    {{-6,-2,-6, 3,-7},{-6,-2, 3,-6,-7},{-2, 4,-2, 3,-3},{ 3,-2, 3,-6,-7},{-7,-3,-7,-7,-8}}, // G C
    {{-6,-6,-2,-6,-7},{-6,-6,-2,-6,-7},{-2,-2, 2,-2,-3},{-6,-6,-2,-6,-7},{-7,-7,-3,-7,-8}}, // G G
    {{-6,-6,-6, 3,-7},{-6,-6, 3,-2,-7},{-2, 3,-2, 4,-3},{ 3,-6, 3,-2,-7},{-7,-7,-7,-3,-8}}, // G U
    {{-7,-7,-7,-7,-3},{-7,-7,-7,-7,-3},{-3,-3,-3,-3, 1},{-7,-7,-7,-7,-3},{-8,-8,-8,-8,-4}}  // G -
  },
  { // A  A  A  A  A    C  C  C  C  C    G  G  G  G  G    U  U  U  U  U    -  -  -  -  -    
    // A  C  G  U  -    A  C  G  U  -    A  C  G  U  -    A  C  G  U  -    A  C  G  U  -    
    {{-2,-6,-6, 3,-7},{-2,-6, 3,-6,-7},{-2, 3,-6, 3,-7},{ 4,-2, 3,-2,-3},{-3,-7,-7,-7,-8}}, // U A
    {{-6,-2,-6,-6,-7},{-6,-2,-6,-6,-7},{-6,-2,-6,-6,-7},{-2, 2,-2,-2,-3},{-7,-3,-7,-7,-8}}, // U C
    {{-6,-6,-2, 3,-7},{-6,-6, 3,-6,-7},{-6, 3,-2, 3,-7},{ 3,-2, 4,-2,-3},{-7,-7,-3,-7,-8}}, // U G
    {{-6,-6,-6,-2,-7},{-6,-6,-6,-2,-7},{-6,-6,-6,-2,-7},{-2,-2,-2, 2,-3},{-7,-7,-7,-3,-8}}, // U U
    {{-7,-7,-7,-7,-3},{-7,-7,-7,-7,-3},{-7,-7,-7,-7,-3},{-3,-3,-3,-3, 1},{-8,-8,-8,-8,-4}}  // U -
  },
  { // A  A  A  A  A    C  C  C  C  C    G  G  G  G  G    U  U  U  U  U    -  -  -  -  -    
    // A  C  G  U  -    A  C  G  U  -    A  C  G  U  -    A  C  G  U  -    A  C  G  U  -    
    {{-3,-7,-7,-7,-8},{-3,-7,-7,-7,-8},{-3,-7,-7,-7,-8},{-3,-7,-7,-7,-8},{ 1,-3,-3,-3,-4}}, // - A
    {{-7,-3,-7,-7,-8},{-7,-3,-7,-7,-8},{-7,-3,-7,-7,-8},{-7,-3,-7,-7,-8},{-3, 1,-3,-3,-4}}, // - C
    {{-7,-7,-3,-7,-8},{-7,-7,-3,-7,-8},{-7,-7,-3,-7,-8},{-7,-7,-3,-7,-8},{-3,-3, 1,-3,-4}}, // - G
    {{-7,-7,-7,-3,-8},{-7,-7,-7,-3,-8},{-7,-7,-7,-3,-8},{-7,-7,-7,-3,-8},{-3,-3,-3, 1,-4}}, // - U
    {{-8,-8,-8,-8,-4},{-8,-8,-8,-8,-4},{-8,-8,-8,-8,-4},{-8,-8,-8,-8,-4},{-4,-4,-4,-4, 0}}  // - -
  }
};

static
const DefaultCostTable::cost_type::value_type
DEFAULT_STATE_COST[3][3] = {
//  M         I         D
  { 0,        0,        0 },// M
  { 0,        0, SHRT_MIN },// I
  { 0, SHRT_MIN,        0 } // D
};

static
const DefaultCostTable::cost_type::value_type
DEFAULT_BRANCH_COST[3][3][3] = {
  {{5,0,0}, {0,0,0}, {0,0,0}},
  {{0,0,0}, {0,0,0}, {0,0,0}},
  {{0,0,0}, {0,0,0}, {0,0,0}},
};

void
PHMMTS::
set_default_cost_table(DefaultCostTable& ct)
{
  const rna_t b[] = { RNA_A, RNA_C, RNA_G, RNA_U, RNA_GAP };
  const match_state_t st[] = { MATCH, INSERT, DELETE };
  DefaultCostTable::cost_type val;

  // for single emits
  for (int i=0; i<N_RNA-1; ++i) {
    for (int j=0; j<N_RNA-1; ++j) {
      if (i<=j) {
	val.set_log_value(DEFAULT_EMIT_COST[i][RNA_GAP][j][RNA_GAP]);
	ct.set_emit_count(b[i], RNA_GAP, b[j], RNA_GAP, val);
      }
    }
    val.set_log_value(DEFAULT_EMIT_COST[i][RNA_GAP][RNA_GAP][RNA_GAP]);
    ct.set_emit_count(b[i], RNA_GAP, RNA_GAP, RNA_GAP, val);
  }

  // for stem emits
  for (int i=0; i<N_RNA-1; ++i) {
    for (int j=0; j<N_RNA-1; ++j) {
      for (int k=0; k<N_RNA-1; ++k) {
	for (int l=0; l<N_RNA-1; ++l) {
	  if (i>k || i==k && j>l)
	    ;
	  else {
	    val.set_log_value(DEFAULT_EMIT_COST[i][j][k][l]);
	    ct.set_emit_count(b[i], b[j], b[k], b[l], val);
	  }
	  
	}
      }
      val.set_log_value(DEFAULT_EMIT_COST[i][j][RNA_GAP][RNA_GAP]);
      ct.set_emit_count(b[i], b[j], RNA_GAP, RNA_GAP, val);
    }
  }

  // for local emits
  for (int i=0; i<N_RNA-1; ++i) {
    ct.set_emit_count(b[i], static_cast<DefaultCostTable::cost_type>(1.0));
  }

  // for state transitions
  for (int i=0; i<3; ++i) {
    for (int j=0; j<3; ++j) {
      val.set_log_value(DEFAULT_STATE_COST[i][j]);
      ct.set_state_count(st[i], st[j], val);
    }
  }
  ct.set_state_count(L_START, MATCH,
		     static_cast<DefaultCostTable::cost_type>(1.0));
  ct.set_state_count(MATCH, L_END,
		     static_cast<DefaultCostTable::cost_type>(1.0));

  // for branch transtion
  for (int i=0; i<3; ++i) {
    for (int j=0; j<3; ++j) {
      for (int k=0; k<3; ++k) {
	if (DEFAULT_BRANCH_COST[i][j][k]!=0) {
	  val.set_log_value(DEFAULT_BRANCH_COST[i][j][k]);
	  ct.set_state_count(st[i], st[j], st[k], val);
	}
      }
    }
  }
}
