/*
 * $Id: defines.h 511 2007-09-18 06:18:08Z satoken $
 *
 * PHMMTS -- an implementation of Pair Hidden Markov Models on Tree Structures,
 *           which is based on "Pair Hidden Markov Models on Tree Structures",
 *           Yasubumi Sakakibara, ISMB 2003.
 *
 * Copyright (C) 2003-2006 Kengo Sato
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef __INC_DEFINES_H__
#define __INC_DEFINES_H__

#include <cassert>

typedef unsigned int uint;

namespace PHMMTS {

  typedef enum {
    GLOBAL_ALIGNMENT,
    LOCAL_ALIGNMENT,
    LOCAL_STRUCTURE,
  } search_type_t;

  typedef enum {
    NOTUSE=0,
    EMPTY,			///< an empty node
    LSINGLE,			///< a node for a single base on the left side
    RSINGLE,			///< a node for a single base on the right side
    STEM,			///< a node for a stem base pair
    BRANCH,			///< a node for a branch node
    START			///< a start node
  } node_type_t;

  typedef enum {
    MATCH=0,
    INSERT=1,
    DELETE=2,
    L_END=3,
    L_START=4,
    N_STATES=5
  } match_state_t;

  typedef enum {
    RNA_A = 0,
    RNA_C = 1,
    RNA_G = 2,
    RNA_U = 3,
    RNA_GAP = 4,
    N_RNA = 5
  } rna_t;

  template <class T>
  struct RNASymbol {
    enum {
      A = RNA_A,
      C = RNA_C,
      G = RNA_G,
      U = RNA_U,
      GAP = RNA_GAP,
    };
  };

  template <>
  struct RNASymbol<char> {
    enum {
      A = 'a',
      C = 'c',
      G = 'g',
      U = 'u',
      GAP = '-',
    };
  };

  struct Position
  {
    int height;
    int state;
    int left;
    int right;

    Position() : height(-1), state(-1), left(-1), right(-1) { }

    Position(int height, int state, int left, int right)
      : height(height), state(state),
	left(left), right(right)
    {
    }

    bool valid() const
    {
      return height>=0 && left>=0 && right>=0;
    }

    bool operator==(const Position& pos) const
    {
      return
	height==pos.height && state==pos.state &&
	left==pos.left && right==pos.right;
    }
    bool operator!=(const Position& pos) const
    {
      return !(*this==pos);
    }
  };

  template <class> class LogValue;
  typedef LogValue<float> cost_t;
}

#endif // __INC_DEFINES_H__

// Local Variables:
// mode: C++
// End:
