/*
 * $Id: example.h 494 2006-08-08 05:29:24Z satoken $
 *
 * PHMMTS -- an implementation of Pair Hidden Markov Models on Tree Structures,
 *           which is based on "Pair Hidden Markov Models on Tree Structures",
 *           Yasubumi Sakakibara, ISMB 2003.
 *
 * Copyright (C) 2003-2006 Kengo Sato
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef __INC_EXAMPLE_H__
#define __INC_EXAMPLE_H__

#include <iostream>
#include <string>
#include "skeletal_base.h"
#include "multiseq.h"

namespace PHMMTS {

  template <class T> class CostTable;

  template <class Seq>
  class ExampleBase
  {
  public:
    typedef typename Seq::value_type rna_t;
    typedef MultiSequence<Seq> MultiSeq;
    typedef typename MultiSeq::value_type multi_rna_t;
    typedef SkeletalTreeNodeBase<multi_rna_t> MultiSkeletalTreeNode;
    typedef SimpleSkeletalTree<MultiSeq,MultiSkeletalTreeNode> MultiSkeletalTree;

  private:
    typedef SimpleSkeletalTree<Seq,SkeletalTreeNodeBase<rna_t> > SeqSkeketalTree;
    MultiSkeletalTree multi_skeletal_;
    SeqSkeketalTree skeletal_;
    Seq unfolded_;
    Seq upstream_;
    Seq downstream_;

  public:
    ExampleBase(const std::string& p, const Seq& f, const Seq& u);

    ExampleBase(const std::string& p, const Seq& f, const Seq& u,
		const Seq& upstream, const Seq& downstream);

    ExampleBase(const std::string& p, const Seq& f, const Seq& u,
		bool not_align);

    const SeqSkeketalTree& skeletal_tree() const
    {
      return skeletal_;
    }

    const Seq& unfolded_seq() const
    {
      return unfolded_;
    }

    template <class T>
    T cost(const CostTable<T>& ct, bool l_opt = false) const;

    template <class T>
    void add_count(CostTable<T>& ct, bool l_opt = false,
		   const T& val = 1.0) const;

    size_t size() const { return multi_skeletal_.size(); }

  public:
    static match_state_t get_state(const MultiSkeletalTreeNode& n);

    enum { LEGAL, IL, IR, DL, DR, ILDR, DLIR, NOT_STEM };
    static int check_ill_stem(const MultiSkeletalTreeNode& n);

    static size_t load_examples(std::istream& in,
				std::list<ExampleBase>& train);

    template <class CT>
    static size_t load_examples(std::istream& in,
				std::list<ExampleBase>& train,
				CT& count_feature, bool l_opt = false);
    
  private:
    static bool is_gap(const typename MultiSeq::value_type& val);
  };

  typedef ExampleBase<std::string> Example;
  typedef ExampleBase<RNASequence> ExampleS;
};

#endif // __INC_EXAMPLE_H__

// Local Variables:
// mode:C++
// End:
