/*
 * $Id: multiseq.cpp 498 2007-03-06 08:44:28Z satoken $
 *
 * PHMMTS -- an implementation of Pair Hidden Markov Models on Tree Structures,
 *           which is based on "Pair Hidden Markov Models on Tree Structures",
 *           Yasubumi Sakakibara, ISMB 2003.
 *
 * Copyright (C) 2003-2006 Kengo Sato
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef __TMPL_MULTI_SEQ_CPP__
#define __TMPL_MULTI_SEQ_CPP__

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <cassert>
#include <string>
#include "multiseq.h"

template < class Sequence >
size_t
MultiSequence<Sequence>::
add_sequence(const Sequence& seq)
{
  if (empty()) resize(seq.size());
  assert(size()==seq.size());
  typename Sequence::const_iterator x;
  iterator y;
  for (x=seq.begin(), y=begin(); x!=seq.end() && y!=end(); ++x, ++y) {
    y->push_back(*x);
  }
  return (*this)[0].size();
}

template < class Sequence >
void
MultiSequence<Sequence>::
nth_sequence(size_t n, Sequence& seq)
{
  seq.clear();
  const_iterator x;
  for (x=begin(); x!=end(); ++x) {
    seq.push_back((*x)[n]);
  }
}

template < class Sequence >
MultiSequence<Sequence>&
MultiSequence<Sequence>::
append(const typename Sequence::value_type& s)
{
  for (const_iterator x=begin(); x!=end(); ++x)
    s.push_back(s);
}


#if 0
#include <iostream>
#include "skeletal.h"
#include "rnatree.h"
#include "rnatree.cpp"
using namespace PHMMTS;
int
main()
{
  typedef MultiSequence<std::string> MultiString;
  typedef SkeletalTreeBase< MultiString > MultiSkeletalTree;
  struct local {
    static bool is_gap(const MultiString::value_type& val)
    {
      MultiString::value_type::const_iterator x;
      for (x=val.begin(); x!=val.end(); ++x) {
	if (*x!='-') return false;
      }
      return true;
    }
  };
  
  const char* paren = "(( )   )(  )";
  const char* seq1  = "ACAGC-UAACGU";
  const char* seq2  = "ACAG-GUAACGU";

  MultiString mstr;
  mstr.add_sequence(std::string(seq1));
  mstr.add_sequence(std::string(seq2));
  MultiSkeletalTree mskel;
  mskel.parse(mstr, std::string(paren), local::is_gap);

  MultiSkeletalTree::node_type r = mskel.root();

  return 0;
}

#endif

#endif // __TMPL_MULTI_SEQ_CPP__
