/*
 * $Id: multiseq.h 482 2006-04-12 09:00:23Z satoken $
 *
 * PHMMTS -- an implementation of Pair Hidden Markov Models on Tree Structures,
 *           which is based on "Pair Hidden Markov Models on Tree Structures",
 *           Yasubumi Sakakibara, ISMB 2003.
 *
 * Copyright (C) 2003-2006 Kengo Sato
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef __INC_MULTI_SEQ_H__
#define __INC_MULTI_SEQ_H__

#include <vector>

template < class Sequence >
class MultiSequence :
  public std::vector< std::vector< typename Sequence::value_type > > 
{
  typedef std::vector< std::vector< typename Sequence::value_type > > base_;

public:
  typedef typename base_::iterator iterator;
  typedef typename base_::const_iterator const_iterator;
  typedef typename base_::reverse_iterator reverse_iterator;
  typedef typename base_::const_reverse_iterator const_reverse_iterator;
  typedef typename base_::value_type value_type;
  typedef typename base_::pointer pointer;
  typedef typename base_::reference reference;
  typedef typename base_::size_type size_type;
  typedef typename base_::difference_type difference_type;
  
public:
  MultiSequence() : base_() { }

  MultiSequence(size_t n) : base_(n) { }

  MultiSequence(const Sequence& seq) : base_()
  {
    add_sequence(seq);
  }

  MultiSequence(const Sequence& seq1, const Sequence& seq2) : base_()
  {
    add_sequence(seq1);
    add_sequence(seq2);
  }

  size_t add_sequence(const Sequence& seq);

  void nth_sequence(size_t n, Sequence& seq);

  MultiSequence& append(const typename Sequence::value_type& s);

  MultiSequence& operator+=(const typename Sequence::value_type& s)
  {
    return append(s);
  }

  using base_::empty;
  using base_::size;
  using base_::begin;
  using base_::end;
};

#include "multiseq.cpp"

#endif // __INC_MULTI_SEQ_H__

// Local Variables:
// mode:C++
// End:
