/*
 * $Id: progress.cpp 511 2007-09-18 06:18:08Z satoken $
 *
 * PHMMTS -- an implementation of Pair Hidden Markov Models on Tree Structures,
 *           which is based on "Pair Hidden Markov Models on Tree Structures",
 *           Yasubumi Sakakibara, ISMB 2003.
 *
 * Copyright (C) 2003-2006 Kengo Sato
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <cstdio>
#include "progress.h"
#include "timersub.h"

using namespace PHMMTS;

ProgressBar::ProgressBar(int columns)
  : cur_(0), from_(0), to_(0), columns_(columns)
{
  timerclear(&start_tm_);
}

void
ProgressBar::start(int from, int to)
{
  from_=from;
  to_=to;
  cur_=from_;
  gettimeofday(&start_tm_, NULL);
  update(0);
}

void
ProgressBar::update(int step /*=1*/)
{
  struct timeval now, t;
  cur_+=step;
  gettimeofday(&now, NULL);
  timersub(&now, &start_tm_, &t);
  float percent = ((float)cur_-from_)/(to_-from_);
  fprintf(stderr, "%3d%% |", (int)(percent*100+0.5));
  int w=(int)((columns_-20)*percent+0.5);
  int i;
  for (i=0; i<w-1; ++i)
    fprintf(stderr, "=");
  if (i==w-1)
    fprintf(stderr, ">");
  for (i=w; i<columns_-20; ++i)
    fprintf(stderr, " ");
  int remain = (int)(t.tv_sec/percent-t.tv_sec);
  if (remain<0) remain=0;
  fprintf(stderr, "| %5ds\r", remain);
}

void
ProgressBar::finish()
{
  struct timeval now, t;
  gettimeofday(&now, NULL);
  timersub(&now, &start_tm_, &t);
  fprintf(stderr, "%3d%% |", 100);
  int w=columns_-20;
  int i;
  for (i=0; i<w-1; ++i)
    fprintf(stderr, "=");
  if (i==w-1)
    fprintf(stderr, ">");
  //for (i=w; i<columns_-20; ++i)
  //  fprintf(stderr, " ");
  fprintf(stderr, "|         \r"/*, t.tv_sec, t.tv_usec/10000*/);
  fprintf(stderr, "\n");
}

