/*
 * $Id: skeletal.cpp 484 2006-07-21 19:00:21Z satoken $
 *
 * PHMMTS -- an implementation of Pair Hidden Markov Models on Tree Structures,
 *           which is based on "Pair Hidden Markov Models on Tree Structures",
 *           Yasubumi Sakakibara, ISMB 2003.
 *
 * Copyright (C) 2003-2006 Kengo Sato
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "skeletal.h"
#include "skeletal_base.h"
#include "skeletal_base.cpp"

using namespace PHMMTS;

namespace PHMMTS {
template < >
void
SimpleSkeletalTree<std::string,
		   SkeletalTreeNodeBase<std::string::value_type> >::
parse(const std::string& str, const std::string& paren)
{
  struct local {
    static bool is_gap(const RNA& val)
    {
      return val=='-';
    }
  };
  RNATree<std::string>* t =
    RNATree<std::string>::parse(str, paren, local::is_gap);
  parse(t);
  t->destroy();
}

template < >
void
SimpleSkeletalTree<RNASequence,
		   SkeletalTreeNodeBase<RNASequence::value_type> >::
parse(const RNASequence& str, const std::string& paren)
{
  struct local {
    static bool is_gap(const RNA& val)
    {
      return val==static_cast<rna_t>(RNASymbol<rna_t>::GAP);
    }
  };
  RNATree< RNASequence >* t =
    RNATree< RNASequence >::parse(str, paren, local::is_gap);
  parse(t);
  t->destroy();
}
};

template
class
SkeletalTreeBase<RNASequence, SkeletalTreeNodeBase<RNASequence::value_type> >;

template
class
SimpleSkeletalTree<RNASequence, SkeletalTreeNodeBase<RNASequence::value_type> >;
